/* sio_cmd.c - support library for command-driven serial communications
 *
 * sio_open()           -  open serial port and set mode
 * sio_open_trace()     -  ditto, with trace file
 * sio_cmd()            -  send a command, retrieve response
 */


//#define DEBUG

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
        
#include "sio_cmd.h"
#define PROMPT1 '$'
#define PROMPT2 '>'

static FILE *tfp = NULL;

/*
 * open <dev>     (/dev/ttySxx port) for serial I/O
 * open <tracef>  for ascii trace dump of serial comm's
 * return fd or -1 on error
 */

int sio_open_trace( char *dev, char *tracef) 
{
  int fd;

  if((fd = sio_open( dev)) < 0)
    return fd;

  if((tfp = fopen( tracef, "w")) == NULL) 
  {
    close( fd);
    return -1;
  }

  return fd;
}
    


/*
 * open <dev>     (/dev/ttySxx port) for serial I/O
 * return fd or -1 on error
 */
int sio_open( char *dev)
{

  struct termios oldtio,newtio;
  int fd;
        
  /* open device for blocking I/O */
  fd = open(dev, O_RDWR | O_NOCTTY);
  //  fd = open(dev, O_RDWR | O_NOCTTY | O_NONBLOCK); 
  if (fd <0) 
  {
    perror(dev);
    return( -1);
  }

  tcgetattr(fd,&oldtio); /* save current port settings */
        
  bzero(&newtio, sizeof(newtio));
  
  ///* new
  //fcntl(fd, F_SETFL, fcntl(fd, F_GETFL, 0) & ~O_NOCTTY);
  //port_init();
  //stat(dev, &stt);
  //portuid = stt.st_uid;
  //portgid = stt.st_gid;
  //if (real_uid != 0) chown(dev, (uid_t)real_uid, (gid_t)real_gid);
  //m_nohang(fd);
  //new */ 

  /* set baud rate, 8 bits, no modem control, enable reading */
  newtio.c_cflag = BAUDRATE | CS8 | CLOCAL | CREAD;
  newtio.c_iflag = 0;   /* raw input */
  newtio.c_oflag = 0;   /* raw output */
        
  /* set input mode (non-canonical, no echo,...) */
  newtio.c_lflag = 0;
         
  /* flush the line and activate new settings */
  tcflush(fd, TCIFLUSH);
  tcsetattr(fd,TCSANOW,&newtio);

  return fd;
}


//
// Send a command string to board, wait for return string terminated by '$' or '>'
// Return pointer to static buffer containing return string or NULL on error
//
char *sio_cmd( int fd, char *s)
{
  char cmd;
  int i, res, nc;
  char rch, *p;
  
  static char buf[SIO_BUF_MAX];

  if( strlen(s) == 0) return NULL;

  write( fd, s, strlen(s));
  write(fd, "\r", 1);

  // if tracing, output the sent string
  if( tfp)
    fprintf( tfp, "Send: %s\n", s);

  // read until we see the prompt
  rch = ' ';
  nc = 0;
  do 
  {
    res = read( fd, &rch, 1);
	 if( res == 1)
    {
      buf[nc++] = rch;
#ifdef DEBUG
		printf("...%d\n", rch);
#endif      
    }
    if( nc == SIO_BUF_MAX) 
    {
      printf("Buffer overflow!\n");
      exit( 1);
    }
  } while( rch != PROMPT1 && rch != PROMPT2);
  buf[nc++] = '\0';


  // if tracing, format the received string, taking special care
  // of control characters
  if( tfp) 
  {
    fprintf( tfp, "Recv: ");
    for( i=0; i<nc; i++) 
    {
      if( !iscntrl( buf[i]))
			putc( buf[i], tfp);
      else 
      {
			switch( buf[i])
			{
				case '\r':
				case '\n':
	  				fprintf( tfp, "\n      ");
	  				break;
				case '\0':
	  				break;
				default:
	  				fprintf( tfp, "?[%02x]", buf[i]);
			}
      }
    }
    fprintf( tfp, "\n");
  }

  if( rch > 0)
    return( buf);
  else
    return( NULL);
}

//
// Send a command string to board, without expecting any interesting return message
//
void sio_shout( int fd, char *s)
{
  char cmd;
  int i, res, nc;
  char rch, *p;
  
  static char buf[SIO_BUF_MAX];

  if( strlen(s) == 0) return;

  write( fd, s, strlen(s));
  write(fd, "\r", 1);

  // if tracing, output the sent string
  if( tfp)
    fprintf( tfp, "Send: %s\n", s);

  // read until we see the prompt
  rch = ' ';
  nc = 0;
  do 
  {
    res = read( fd, &rch, 1);
	 if( res == 1)
    {
      buf[nc++] = rch;
#ifdef DEBUG
		printf("...%d\n", rch);
#endif      
    }
    if( nc == SIO_BUF_MAX) 
    {
      printf("Buffer overflow!\n");
      exit( 1);
    }
  } while( rch != '\r');
  buf[nc++] = '\0';


  // if tracing, format the received string, taking special care
  // of control characters
  if( tfp) 
  {
    fprintf( tfp, "Recv: ");
    for( i=0; i<nc; i++) 
    {
      if( !iscntrl( buf[i]))
			putc( buf[i], tfp);
      else 
      {
			switch( buf[i])
			{
				case '\r':
				case '\n':
	  				fprintf( tfp, "\n      ");
	  				break;
				case '\0':
	  				break;
				default:
	  				fprintf( tfp, "?[%02x]", buf[i]);
			}
      }
    }
    fprintf( tfp, "\n");
  }
}


// dump a string in hex for debug

void dump_string( char *s)
{
  char *p;

  for( p=s; *p; ++p) 
  {
    printf("[%02x] '%c'\n", *p, *p<0x20 ? '-' : *p);
  }
}
